#!/usr/bin/perl -w

## File: sockclient.perl
## Author: Bryan Jurish <moocow@ling.uni-potsdam.de>
## Description: Example TCP socket client

use IO::Socket::INET;

##----------------------------------------------------------------------
## GLOBALS
##----------------------------------------------------------------------

## $PROTO
##  + connection protocol
our $PROTO = 'tcp';

## $TYPE
##  + connection type
our $TYPE = SOCK_STREAM;

## $HOST
##  + (default) remote hostname or IP
our $HOST = 'localhost';

## $PORT
##  + (default) port number or service name to connect to
our $PORT = 'echo';


##----------------------------------------------------------------------
## Signal Handlers
##----------------------------------------------------------------------
foreach my $sig (qw(KILL INT PIPE)) {
  $SIG{$sig} = sub { die("$0: received SIG$sig -- bailing out!\n"); };
}

##----------------------------------------------------------------------
## MAIN
##----------------------------------------------------------------------

##-- Read command-line argument: '[HOST][:PORT][@PROTO][/TYPE]'
if (@ARGV) {
  $dest = shift(@ARGV);
  if ($dest =~ /^\s*([^\:]*)(?:\:([^\@]*))?(?:\@([^\/]*))?(?:\/(\S*))?\s*$/) {
    ($host,$port,$proto,$type) = ($1,$2,$3,$4);
    $HOST = $host if ($host);
    $PORT = $port if ($port);
    $PROTO = $proto if ($proto);
    if ($type) {
      $TYPE = SOCK_DGRAM if ($type eq 'dgram');
      $TYPE = SOC_STREAM if ($type eq 'stream');
    }
  }
}

##-- create the client socket
$c_sock = IO::Socket::INET->new(
				PeerAddr => $HOST,
				PeerPort => $PORT,
				Proto    => $PROTO,
				Type     => $TYPE,
			       )
  or die("$0: could not open socket to $HOST:$PORT\@$PROTO/$TYPE: $!");

##-- read input data
while (<>) {
  chomp;
  ##-- special command 'get' reads a line from the socket
  if ($_ eq 'get') {
    ##-- get a line from the socket
    $got = <$c_sock>;
    if (!defined($got)) {
      warn("$0: no data available for 'get'");
    }
    else {
      print "> ",  $got;
    }
  }
  ##-- everything else gets written to the socket
  else {
    die("$0: socket error: $!") if ($c_sock->error);
    $c_sock->print($_, "\015\012"); ##-- use internet line terminators
  }
}
