#!/usr/bin/perl -w

## File: sockserver.perl
## Author: Bryan Jurish <moocow@ling.uni-potsdam.de>
## Description: Example TCP socket server


use IO::Socket::INET;

##----------------------------------------------------------------------
## GLOBALS
##----------------------------------------------------------------------

## $PROTO
##  + connection protocol
our $PROTO = 'tcp';

## $TYPE
##  + connection type
our $TYPE = SOCK_STREAM;

## $HOST
##  + (default) local hostname or IP to bind
our $HOST = inet_ntoa(INADDR_ANY);

## $PORT
##  + (default) port number or service name to bind
our $PORT = 4567;

## $QSIZE
##  + client queue length
our $QSIZE = 2;

## $TIMEOUT
##  + timeout for various operations (seconds)
our $TIMEOUT = 5;


##----------------------------------------------------------------------
## Signal Handlers
##----------------------------------------------------------------------
foreach my $sig (qw(KILL INT PIPE)) {
  $SIG{$sig} = sub { die("$0: received SIG$sig -- bailing out!\n"); };
}

##----------------------------------------------------------------------
## MAIN
##----------------------------------------------------------------------

##-- Read command-line argument: '[HOST][:PORT][@PROTO][/TYPE][+QSIZE][-TIMEOUT]'
if (@ARGV) {
  $dest = shift(@ARGV);
  if ($dest =~ /^\s*([^\:]*)(?:\:([^\@]*))?(?:\@([^\/]*))?(?:\/([^\+]*))?(?:\+([^\-]*))?(?:\-(\S*))?\s*$/) {
    ($host,$port,$proto,$type,$qsize,$timeout) = ($1,$2,$3,$4,$5,$6);
    $HOST = $host if ($host);
    $PORT = $port if ($port);
    $PROTO = $proto if ($proto);
    if ($type) {
      $TYPE = SOCK_DGRAM if ($type eq 'dgram');
      $TYPE = SOC_STREAM if ($type eq 'stream');
    }
    $QSIZE = $qsize if ($qsize);
    $TIMEOUT = $timeout if ($timeout);
  }
}

##-- create the server socket
$s_sock = IO::Socket::INET->new(
    LocalAddr  => $HOST,
    LocalPort  => $PORT,
    Proto      => $PROTO,
    Type       => $TYPE,
    Listen     => $QSIZE,
    ReuseAddr  => 1,
    Timeout    => $TIMEOUT,
  ) or die("$0: could not create socket $HOST:$PORT\@$PROTO/$TYPE+$QSIZE-$TIMEOUT: $!");

##-- listen for connections
while (1) {
  while ($c_sock = $s_sock->accept()) {
    ##-- report the connection
    $peerhost = $c_sock->peerhost();
    print STDERR "$0: connect from $peerhost\n";

    ##-- echo input
    while (<$c_sock>) {
      $_ =~ s/\015*\012?$//;
      if ($_ eq 'bye') {
	##-- close connection on 'bye'
	$c_sock->print("Goodbye!\015\012");
	$c_sock->close();
	last;
      }
      $c_sock->print($_, "\015\012");  ##-- use internet line terminators
    }
    print STDERR "$0: client $peerhost disconnected.\n";
  }
}
