#!/usr/bin/perl -w

##=============================================================
## File: wc.perl
## Author: Bryan Jurish
## Description:
##   + display line and character counts for input text file(s)
##=============================================================

##=============================================================
## GLOBALS
##=============================================================

## $VERSION
##   + program version
our $VERSION = 0.01;

## $filename
##   + current input filename
our $filename = undef;

## $lines_f
##   + number of lines in current file
our $lines_f = 0;

## $lines_total
##   + total number of lines in all files
our $lines_total = 0;

## $chars_f
##   + number of characters in current file
our $chars_f = 0;

## $chars_total
##   + total number of characters in all files
our $chars_total = 0;


##=============================================================
## Subroutines
##=============================================================

## ($lines,$chars) = wc_file($filename)
##  + counts and returns the number of lines and characters in $filename
##  + adds counted number of lines to global $lines_total
##    and counted number of characters to global $chars_total
sub wc_file {
  my $filename = shift;
  my ($lines, $chars) = (0,0);  ##-- local counter variables

  open(WCFILE,"<$filename")
    or die("$0: open failed for file '$filename': $!");

  ##-- count 'em up
  while (<WCFILE>) {
    $lines++;
    $chars += length($_);
  }
  $lines_total += $lines;
  $chars_total += $chars;

  close(WCFILE);
  return ($lines,$chars);
}

##=============================================================
## MAIN
##=============================================================
push(@ARGV,'-') if (!@ARGV);

foreach $filename (@ARGV) {
  ($lines_f, $chars_f) = wc_file($filename);
  print "$filename:\t $lines_f lines,\t $chars_f characters\n";
}

print "TOTAL:\t $lines_total lines,\t $chars_total characters\n";
